//
//  RSVRSDKConfig.h
//  Lianjia_Beike_VR
//
//  Created by 林飞阳 on 2020/2/17.
//

#import <Foundation/Foundation.h>
#import "RSVRShareBaseModel.h"

NS_ASSUME_NONNULL_BEGIN


typedef NSString * _Nullable (^RSVRConfigStringValueBlock)(void);
typedef void (^RSVRSchemeFinishCallback)(NSString *);

typedef NS_ENUM(NSInteger, RSVRSDKEnv) {
    RSVRSDKEnvProduction = 0, // 线上环境
    RSVRSDKEnvSandbox, // 沙盒环境
};

typedef NS_ENUM(NSInteger, RSVROpenErrorType) {
    RSVROpenErrorTypeInvalidateUrl = 1,  // url 解析失败
};

@protocol RSVRSDKDelegate <NSObject>

@optional

/// H5 页面点击分享时调用
/// @param param  H5 页面传过来的分享参数
/// @note 如果业务实现了这个方法, 分享将由业务来控制
- (void)shareWithParam:(NSString *)param;

/// 当用户点击分享 item 回调
/// @param shareModel 分享 item 数据
- (void)didShareItemClick:(RSVRShareBaseModel *)shareModel;

/// VR webview 内有一些页面需要登录才可以进入,
/// 需要在这个代理方法里面实现唤起登录界面.
/// @param didUserLoginBlock 当用户登录成功之后, 你需要主动调用这个 block
- (void)actionLogin:(dispatch_block_t)didUserLoginBlock;

/// webview 页面跳转回调
/// @param router 跳转 router
/// @note 业务方可实现这个回调, 接收 H5 跳转页面的请求
- (void)onActionRouter:(NSString *)router;

/// 需要实现这个方法来处理一些 VR SDK 没有处理的 H5 scheme, 比如: web/compaign
/// @param urlString sscheme url
/// @param finishBlock scheme 回调
- (void)actionURL:(NSString *)urlString withFinishBlock:(RSVRSchemeFinishCallback _Nullable)finishBlock;

/// 打开 VR 失败回调
/// @param reason 失败原因
/// @param userInfo 失败信息, {"url": "打开失败的 url"}
- (void)failToOpenVR:(RSVROpenErrorType)reason userInfo:(NSDictionary *)userInfo;

/// 退出VR 页面回调
- (void)onExitVR;

@end

# pragma mark -

@interface RSVRAppInfo : NSObject

/// appId, 用于带看语音验签, 需要向如视后端申请
/// @note [必选]
@property (nonatomic, copy) NSString *appId;

/// appSecret
/// @note [必选]
@property (nonatomic, strong) NSString *appSecret;

/// jsBridge 所调用 sscheme 的前缀, 用于 VR H5 跟客户端之间的通信
/// @note 不要包含除了字母以外的字符
/// @note [必选]
@property (nonatomic, copy) NSString *scheme;

/// webView 的 UA 标识, VR H5 会根据这个标识识别不同的业务方
/// @note [必选]
@property (nonatomic, copy) NSString *userAgent;

@end

# pragma mark -

@interface RSVRSDKConfig : NSObject

@property (nonatomic, strong) RSVRAppInfo *appInfo;

/// ucid
/// @note [必选]
@property (nonatomic, copy) RSVRConfigStringValueBlock userId;

/// 设置用户名
@property (nonatomic, copy) RSVRConfigStringValueBlock userName;

/// VR SDK 的环境
@property (nonatomic, assign) RSVRSDKEnv env;

/// 当前登录用户的 token
/// 当前登录用户的 token, 用于 VR 语音带看
/// @note [必选]
@property (nonatomic, copy) RSVRConfigStringValueBlock accessToken;

/// VR webview 默认的 loading logo 图片
/// @note [可选]
@property (nonatomic, strong) UIImage *defaultLoadingLogoImage;

/// 往 VR webview 里面注入数据, 这个数据会通过 jsBridge 透传到 H5
@property (nonatomic, copy) NSDictionary * _Nullable (^extData)(void);

@property (nonatomic, weak) id<RSVRSDKDelegate> configDelegate;

+ (instancetype)shareInstance;

@end

NS_ASSUME_NONNULL_END
